"""
The Place comparator list different methods to
compare a current place to a memorised place or
memorised places.
"""
import numpy as np
from Scene_processing import is_ibpc, is_obpc


def imagediff(current, memory):
    """Compute the root mean square difference between
the current and memorised place code

    :param current: current place code
    :param memory: memorised place code
    :returns: the image difference
    :rtype: float

    ..ref: Zeil, J., 2012. Visual homing: an insect perspective.
           Current opinion in neurobiology

    """
    assert isinstance(current, np.ndarray),\
        'current place code should be a numpy array'
    assert isinstance(memory, np.ndarray),\
        'memory place code should be a numpy array'
    assert np.all(current.shape == memory.shape),\
        'memory and current place code should have the same shape'
    diff = np.power(current - memory, 2)
    if is_ibpc(current):
        return np.sqrt(diff.mean(axis=0).mean(axis=1))
    elif is_obpc(current):
        return np.sqrt(diff.mean(axis=0).mean(axis=0))
    else:
        raise TypeError('place code is neither an ibpc nor obpc')


def rot_imagediff(current, memory):
    """Compute the rotational image difference between
the current and memorised place code.

    :param current: current place code
    :param memory: memorised place code
    :returns: the rotational image difference
    :rtype: (np.ndarray)

    ..ref: Zeil, J., 2012. Visual homing: an insect perspective.
           Current opinion in neurobiology
    ..note: assume that the image is periodic along the x axis
           (the left-right axis)
    """
    assert is_ibpc(current),\
        'The current and memory place code should be image based'
    ridf = np.zeros(current.shape[1])
    for azimuth_i in range(0, current.shape[1]):
        rot_im = np.roll(current, azimuth_i, axis=1)
        rot_im[azimuth_i] = imagediff(rot_im, memory)
    return ridf
