import processing.constants as prc
import numpy as np


def is_ibpc(place_code):
    """Test if a place code is image based

    :param place_code: a place-code
    :returns: True if image based place-code
    :rtype: bool

    """
    toreturn = isinstance(place_code, np.ndarray)
    toreturn = toreturn and (len(place_code.shape) ==
                             len(prc.__ibpc_indeces__))
    return toreturn


def is_obpc(place_code):
    """Test if a place code is ommatidia based

    :param place_code: a place-code
    :returns: True if ommatidia based place-code
    :rtype: bool

    """
    toreturn = isinstance(place_code, np.ndarray)
    toreturn = toreturn and (len(place_code.shape) ==
                             len(prc.__obpc_indeces__))
    return toreturn


def ibs_to_obs(scene, eye_map):
    """Convert an image based scene to an ommatidium based scene.

    :param scene: The scene to be converted
    :param eye_map: The eye_map to use
    :returns: (obs_scene,ommatidia_map)
    :rtype: (np.ndarray,np.ndarray)
    """
    assert is_ibpc(scene),\
        'scene should be an ibs scene'
    assert isinstance(eye_map, np.ndarray), 'eye_map should be a numpy array'
    assert len(eye_map.shape) == len(prc.__eye_indeces__),\
        'eye_map should have {} dimensions to be an ibs scene'.format(
            len(prc.__eye_indeces__))
    for index_name in ['elevation', 'azimuth']:
        index = prc.__ibpc_indeces__[index_name]
        assert eye_map.shape[index] == scene.shape[index],\
            'eye_map and scene should have the same number of {}'.format(
                index_name)
    obs_size = (scene.shape[prc.__ibpc_indeces__['elevation']] *
                scene.shape[prc.__ibpc_indeces__['azimuth']],
                scene.shape[prc.__ibpc_indeces__['channel']],
                scene.shape[prc.__ibpc_indeces__['component']])
    obs_scene = scene.reshape(obs_size)
    omm_size = (eye_map.shape[prc.__ibpc_indeces__['elevation']] *
                eye_map.shape[prc.__ibpc_indeces__['azimuth']],
                eye_map.shape[prc.__ibpc_indeces__['component']])
    ommatidia_map = eye_map.reshape(omm_size)
    return (obs_scene, ommatidia_map)


def cartesian_to_spherical(x, y, z):
    radius = np.sqrt(x**2 + y**2 + z**2)
    elevation = np.arctan2(z, np.sqrt(x**2 + y**2))
    azimuth = np.arctan2(y, x)
    spherical = np.zeros_like(x)
    spherical = np.tile(spherical[..., np.newaxis], (3,))
    spherical[..., prc.__spherical_indeces__['elevation']] = elevation
    spherical[..., prc.__spherical_indeces__['azimuth']] = azimuth
    spherical[..., prc.__spherical_indeces__['radius']] = radius
    return spherical


def spherical_to_cartesian(elevation, azimuth, radius=1):
    cartesian = np.zeros_like(elevation)
    cartesian = np.tile(cartesian[..., np.newaxis], (3,))
    cartesian[..., prc.__cartesian_indeces__['x']] = np.cos(
        elevation) * np.cos(azimuth)
    cartesian[..., prc.__cartesian_indeces__['y']] = np.cos(
        elevation) * np.sin(azimuth)
    cartesian[..., prc.__cartesian_indeces__['z']] = np.sin(elevation)
    cartesian = radius * cartesian
    return cartesian
