"""
    How to test the script:
    -----------------------
    >>> blender test.blend --background --python Cyberbee.py

    :Author: Olivier Bertrand (olivier.bertrand@uni-bielefeld.de)
    :Parent module: Scene_rendering
"""
import bpy
import numpy as np
import tempfile
import os


class Cyberbee():
    """
    Cyberbee is a small class binding python with blender.
    With Cyberbee one can move the bee to a position, and render what
    the bee see at this position.

    The Bee eye is a panoramic camera with equirectangular projection
    The light rays attaining the eyes are filtered with a gaussian.
    """

    def __init__(self):
        """Initialise the Cyberbee"""
        # Rendering engine needs to be Cycles to support panoramic
        # equirectangular camera
        bpy.context.scene.render.engine = 'CYCLES'
        bpy.context.scene.render.layers["RenderLayer"].use_pass_z = True
        # Look for object camera
        camera_found = False
        for obj in bpy.context.scene.objects:
            if obj.type == 'CAMERA':
                self.camera = obj
                camera_found = True
                break
        assert camera_found, 'The blender file does not contain a camera'
        # The bee eye is panoramic, and with equirectangular projection
        self.camera.data.type = 'PANO'
        self.camera.data.cycles.panorama_type = 'EQUIRECTANGULAR'
        # Filtering props
        bpy.context.scene.cycles.filter_type = 'GAUSSIAN'
        # Call all set function with default values
        self.set_camera_rotation_mode()
        self.set_camera_fov()
        self.set_camera_gaussian_width()
        self.set_camera_resolution()
        self.set_cycle_samples()
        # switch on nodes
        # Create render link to OutputFile with Image and Z buffer
        bpy.context.scene.use_nodes = True
        scene = bpy.context.scene
        nodes = scene.node_tree.nodes

        render_layers = nodes['Render Layers']
        output_file = nodes.new("CompositorNodeOutputFile")
        output_file.format.file_format = "OPEN_EXR"
        output_file.file_slots.remove(output_file.inputs[0])
        tmp_fileoutput = dict()
        tmp_fileoutput['Image'] = 'Image'
        tmp_fileoutput['Depth'] = 'Depth'
        tmp_fileoutput['Folder'] = tempfile.TemporaryDirectory().name
        tmp_fileoutput['ext'] = '.exr'
        output_file.file_slots.new(tmp_fileoutput['Image'])
        output_file.file_slots.new(tmp_fileoutput['Depth'])
        output_file.base_path = tmp_fileoutput['Folder']
        scene.node_tree.links.new(
            render_layers.outputs['Image'],
            output_file.inputs['Image']
        )
        scene.node_tree.links.new(
            render_layers.outputs['Z'],
            output_file.inputs['Depth']
        )
        self.tmp_fileoutput = tmp_fileoutput

    def set_camera_rotation_mode(self, mode='XYZ'):
        """change the camera rotation mode

        :param mode: the mode of rotation for the camera see blender doc
                     (default: 'XYZ').
        :type mode: a string
        .. seealso: blender bpy.data.scenes["Scene"].camera.rotation_mode
        """
        bpy.data.scenes["Scene"].camera.rotation_mode = mode

    def get_camera_rotation_mode(self):
        """get the current camera rotation mode

        :returns: the mode of rotation used by the camera
        :rtype: string
        """
        return bpy.data.scenes["Scene"].camera.rotation_mode

    def set_cycle_samples(self, samples=30):
        """change the samples for rendering with cycle

        :param samples: the number of samples to use when rendering images
        :type samples: int
        """
        bpy.context.scene.cycles.samples = samples

    def get_cycle_samples(self):
        """get the samples for rendering with cycle

        :returns: the number of samples used for the rendering
        :rtype: int
        """
        return bpy.context.scene.cycles.samples

    def set_camera_fov(self, latmin=-90, latmax=+90,
                       longmin=-180, longmax=+180):
        """change the field of view of the panoramic camera

        :param latmin: minimum latitude (in deg)
        :type latmin: float
        :param latmax: maximum latitude (in deg)
        :type latmax: float
        :param longmin: minimum longitude (in deg)
        :type longmin: float
        :param longmin: maximum longitude (in deg)
        :type longmin: float
        """
        assert self.camera.data.type == 'PANO', 'Camera is not panoramic'
        assert self.camera.data.cycles.panorama_type == 'EQUIRECTANGULAR',\
            'Camera is not equirectangular'
        self.camera.data.cycles.latitude_min = np.deg2rad(latmin)
        self.camera.data.cycles.latitude_max = np.deg2rad(latmax)
        self.camera.data.cycles.longitude_min = np.deg2rad(longmin)
        self.camera.data.cycles.longitude_max = np.deg2rad(longmax)

    def get_camera_fov(self):
        """get fov of camera

        :returns: the field of view of the camera as min/max,longitude/latitude
               in degrees
        :rtype: dict
        """
        assert self.camera.data.type == 'PANO', 'Camera is not panoramic'
        assert self.camera.cycles.panorama_type == 'EQUIRECTANGULAR',\
            'Camera is not equirectangular'
        fov = dict()
        fov['latitude_min'] = np.rad2ged(self.camera.data.cycles.latitude_min)
        fov['latitude_max'] = np.rad2ged(self.camera.data.cycles.latitude_max)
        fov['longitude_min'] = np.rad2ged(
            self.camera.data.cycles.longitude_min)
        fov['longitude_max'] = np.rad2ged(
            self.camera.data.cycles.longitude_max)
        return fov

    def set_camera_gaussian_width(self, gauss_w=1.5):
        """change width of the gaussian spatial filter

        :param gauss_w: width of the gaussian filter
        :type gauss_w: float
        """
        bpy.context.scene.cycles.filter_width = gauss_w

    def get_camera_gaussian_width(self, gauss_w=1.5):
        """get width of the gaussian spatial filter

        :returns: the width of the gaussian filter
        :rtype: float
        """
        return bpy.context.scene.cycles.filter_width

    def set_camera_resolution(self, resolution_x=360, resolution_y=180):
        """change the camera resolution (nb of pixels)

        :param resolution_x: number of pixels along the x-axis of the camera
        :type resolution_x: int
        :param resolution_y: number of pixels along the y-axis of the camera
        :type resolution_y: int
        """
        bpy.context.scene.render.resolution_x = resolution_x
        bpy.context.scene.render.resolution_y = resolution_y
        bpy.context.scene.render.resolution_percentage = 100

    def get_camera_resolution(self):
        """return camera resolution (x,y)

        :returns: the resolution of the camera along (x-axis,y-axis)
        :rtype: (int,int)
        """
        resolution_x = bpy.context.scene.render.resolution_x
        resolution_y = bpy.context.scene.render.resolution_y
        return resolution_x, resolution_y

    def update(self, posorient):
        """assign the position and the orientation of the camera.

        :param posorient: is a 1x6 vector continaing:
             x,y,z, angle_1, angle_2, angle_3,
             here the angles are euler rotation around the axis
             specified by scene.camera.rotation_mode
        :type posorient: 1x6 double array
        """
        assert len(posorient) == 6, 'posorient should be a 1x6 double array'
        self.camera.location = posorient[:3]
        self.camera.rotation_euler = posorient[3:]
        # Render
        bpy.ops.render.render()

    def get_image(self):
        """return the last rendered image as a numpy array

        :returns: the image (height,width,4)
        :rtype: a double numpy array

        .. note: A temporary file will be written on the harddrive,
                 due to API blender limitation
        """
        # save image as a temporary file, and then loaded
        # sadly the rendered image pixels can not directly be access
        filename = os.path.join(self.tmp_fileoutput['Folder'],
                                self.tmp_fileoutput['Image'] + '0001')
        im_width, im_height = self.get_camera_resolution()
        im = bpy.data.images.load(filename)
        pixels = np.array(im.pixels)
        # im=PIL.Image.open(filename)
        # pixels=np.asarray(im)
        pixels = pixels.reshape([im_height, im_width, 4])
        return pixels

    def get_distance(self):
        """return the last rendered distance map as a numpy array

        :returns: the distance map (height, width)
        :rtype: a double numpy array

        .. note: A temporary file will be written on the harddrive,
                 due to API blender limitation
        """
        # save image as a temporary file, and then loaded
        # sadly the rendered image pixels can not directly be access
        filename = os.path.join(self.tmp_fileoutput['Folder'],
                                self.tmp_fileoutput['Depth'] + '0001')
        im_width, im_height = self.get_camera_resolution()
        im = bpy.data.images.load(filename)
        distance = np.array(im.pixels)
        # im=PIL.Image.open(filename)
        # distance=np.asarray(im)
        distance = distance.reshape([im_height, im_width, 4])
        distance = distance[:, :, 0]
        return distance


if __name__ == "__main__":
    # Initiate the Cyberbee
    mybee = Cyberbee()
    frames_per_revolution = 5.0
    step_size = 2 * np.pi / frames_per_revolution
    posorients = np.zeros((frames_per_revolution, 6))
    posorients[:, 0] = np.sin(np.arange(frames_per_revolution) * step_size) * 5
    posorients[:, 1] = np.cos(np.arange(frames_per_revolution) * step_size) * 5
    for frame_i, posorient in enumerate(posorients):
        mybee.update(posorient)
        # Test image
        image = mybee.get_image()
        # Test distance
        distance = mybee.get_distance()
    print('Cyberbee OK')
